package examples.ejb20.relationships.many2many;

import java.rmi.RemoteException;
import java.util.Collection;
import java.util.Iterator;
import java.util.Hashtable;
import java.util.Properties;
import java.util.Vector;
import java.util.Iterator;

import javax.ejb.CreateException;
import javax.ejb.DuplicateKeyException;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

/**
 * This class demonstrates CMP relationships and EJB-QL by performing the following:
 * <p>
 * <ul><li>Create 5 seperate accounts and customers, with different initial balances and type.
 * <li>Establish relationships between beens.
 * <li>Print out relationships.
 * <li>Find all the accounts with a balance greater than a specific value.
 * <li>Find the accounts with a balance of zero.
 * <li>Find the accounts with a null type.
 * </ul>
 * Finally, remove all newly created accounts.
 * <p>
 * This class also illustrates how to lookup an EJB home in the JNDI tree. 
 * 
 * @author Copyright (c) 1998-2000 by BEA Systems, Inc. All Rights Reserved.
 */
public class Client {

  private String url;
  private AccountHome accountHome;
  private CustomerHome customerHome;

  public Client(String url) 
    throws NamingException {

    this.url       = url;

    accountHome = lookupAccountHome();
    customerHome = lookupCustomerHome();
  }

  /**
   * Runs this example from the command line. Example:
   * <p>
   * <tt>java examples.ejb20.relationships.many2many.Client "t3://localhost:7001" 10020</tt>
   * <p>
   * The parameters are optional, but if any are supplied,
   * they are interpreted in this order:
   * <p>
   * @param url               URL such as "t3://localhost:7001" of Server
   */
  public static void main(String[] args) {
    System.out.println("\nBeginning examples.ejb20.relationships.many2many.Client...\n");

    String url       = "t3://localhost:7001";
 
    if (args.length == 1) {
      url = args[0];
    } else if (args.length > 1) {
      System.out.println("Usage: examples.ejb20.relationships.many2many.Client WebLogicURL");
      return;
    }

    Client client = null;

    try {
      client = new Client(url);
    } catch (NamingException ne) {
      log("Unable to look up the beans home: " + ne.getMessage());
      System.exit(1);
    }

    try {
      client.runExample();
    } catch (Exception e) {
      log("There was an exception while creating and using the Accounts.");
      log("This indicates that there was a problem communicating with " +
          "the server: "+e);
    }

    System.out.println("\nEnd examples.ejb20.relationships.many2many.Client...\n");
  }


  /**
   * Create 5 new accounts and customers, with different initial balances and type.
   * Find all the accounts with a balance greater than a specific value.
   * When finished, remove the new accounts.
   */
  public void runExample()
     throws CreateException, RemoteException, FinderException,
            RemoveException
  {
    int numBeans = 5;

    Account [] accounts = new Account [numBeans];
    Customer [] customers = new Customer [numBeans];

    // Create accounts with varying balances and some null accountTypes.
    log("\nCreate Accounts and customers:");
    for (int i=0; i<numBeans; i++) {
      // if (Math.IEEEremainder(i, 2) == 0)
      if (i % 2 == 0) {
        // Create a few of beans with a null accountType 
        // (just so we can find them later.)
        accounts [i] = findOrCreateAccount("ID: "+i, i * 1000, null);
      } else {
        accounts [i] = findOrCreateAccount("ID: "+i, i * 1000, "Savings");
      }

      try {
	customers[i] = createCustomer("customer_"+i);
      } catch(DuplicateKeyException dke) {
        // This won't prevent us from continuing the example
        log("A customer with name customer_" + i + " already exists!");
      } catch(Exception e) {
        log("Error creating account: " + e.getMessage());
        e.printStackTrace();
        System.exit(1);
      }
    }

    log("\nEstablish relationships:");
    for (int i=0; i<numBeans; i++) {
      for (int x=0; x<numBeans; x++) {
        try {
          // add all acounts to customer_0
          log("Add account "+ accounts[i].getAccountId() + " to " + 
            customers[x].getName());
          customers[x].addAccount(accounts[i]);
          // add all customers to the account ID 0
          log("Add customer "+ customers[i].getName() + " to " +
            accounts[x].getAccountId());
          accounts[x].addCustomer(customers[i]);
        } catch(Exception e) {
          log("Error setting account: " + e.getMessage());
          e.printStackTrace();
          System.exit(1);
        }
      }
    }

    log("\nPrint Customers:");
    // print out the customers
    for (int i=0; i<numBeans; i++) {
      log("Customer: " + customers[i].getName() + " has accounts:");
       Iterator iter = customers[i].getAllAccounts().iterator();
       while (iter.hasNext()) {
         Account acct = (Account)iter.next();
         log("\t" + acct.getPrimaryKey() + " with balance " +
         acct.balance());
       }
    }

    log("\nPrint Accounts:");
   // print out the accounts
    for (int i=0; i<numBeans; i++) {
      log("Account " + accounts[i].getAccountId() + " has customers:");
       Iterator iter = accounts[i].getAllCustomers().iterator();
       while (iter.hasNext()) {
         Customer cust = (Customer)iter.next();
         log("\t" + cust.getPrimaryKey());
       }
    }

    // find all accounts with a balance > 2000
    findBigAccounts(2000.0);
    // find the first account with a balance of 0
    findAccountWithZeroBalance();
    // find all accounts with a null balance
    findNullAccounts();

    // Remove our accounts
    log("Removing beans...");
    for (int i=0; i<numBeans; i++) {
         accounts[i].remove();
         customers[i].remove();
    }
  }

  /**
   * List the first account with a zero balance.
   * This finder illustrates a Finder Method that only returns one record.
   */
  private void findAccountWithZeroBalance() 
    throws FinderException, RemoteException
  {
    log("\nQuerying for an account with zero balance...");
    try {
      Account zeroBalance = (Account) 
	PortableRemoteObject.narrow(accountHome.findAccount(0), Account.class);
      // Account zeroBalance = home.findAccount(0);
      log("Account " + zeroBalance.getPrimaryKey() + "; balance is zero");
    } catch(ObjectNotFoundException onfe) {
      log("No Account was found with a balance of zero");
    }
  }

  /**
   * List all accounts with a null balance.
   * This finder illustrates a Finder Method that returns a Collection.
   */
  private void findNullAccounts()
    throws RemoteException, FinderException
  {
    log("\nQuerying for accounts with a null account type");
    Collection col = accountHome.findNullAccounts();

    Iterator it = col.iterator();
    if(! it.hasNext()) {
      log("No accounts were found with a null account type");
    }

    while (it.hasNext()) {
      Account nullAccount= (Account)it.next();
      log("Account " + nullAccount.getPrimaryKey() + "; account type is " + nullAccount.accountType());
    }
  }

   /**
    * List all accounts with a balance greater than the parameterized amount.
    * This finder illustrates a Finder Method that returns an Enumeration.
    */
  private void findBigAccounts(double balanceGreaterThan) 
    throws RemoteException, FinderException
  {
    log("\nQuerying for accounts with a balance greater than " +
      balanceGreaterThan + "...");

    Collection col = accountHome.findBigAccounts(balanceGreaterThan);
    Iterator it = col.iterator();

    if(! it.hasNext()) {
      log("No accounts were found!");
    }

    while (it.hasNext()) {
      Account bigAccount= (Account)it.next();

      log("Account " + bigAccount.getPrimaryKey() + 
        "; balance is $" + bigAccount.balance());
    }
  }


  /**
   * If there already exists an account for this id, we will return 
   * it.  Otherwise we will create a new account.
   */
  private Account findOrCreateAccount(String id, double balance, String accountType)
    throws CreateException, FinderException, RemoteException
  {
    Account remote = null;
    try {
      remote = (Account) PortableRemoteObject.narrow(accountHome.findByPrimaryKey(id), 
        Account.class);
      // remote = home.findByPrimaryKey(id);
    } catch (ObjectNotFoundException onfe) {
      // the account id does not yet exist so create it.
      remote = createAccount(id, balance, accountType);
    }
    return remote;
  }


  /**
   * Create a new account with the given id and balance
   */
  private Account createAccount(String id, double balance, String accountType) 
    throws CreateException, RemoteException 
  {
    log ("Creating account " + id + " with a balance of " + 
      balance + " account type " + accountType + "...");

    Account ac = (Account) PortableRemoteObject.narrow(
      accountHome.create(id, balance, accountType),
      Account.class);

   log("Account " + id + " successfully created");

   return ac;
  }

  /**
   * Create a new customer with the given name
   */
  private Customer createCustomer(String name) 
    throws CreateException, RemoteException 
  {
    log ("Creating customer " + name + "...");

    Customer cust = (Customer) PortableRemoteObject.narrow(
      customerHome.create(name), Customer.class);

   log("Customer " + name + " successfully created");

   return cust;
  }

  /**
   * Look up the account bean's home interface using JNDI.
   */
  private AccountHome lookupAccountHome()
    throws NamingException
  {
    Context ctx = getInitialContext();

    try {
      Object home = (AccountHome) ctx.lookup("many2many.AccountHome");
      return (AccountHome) PortableRemoteObject.narrow(home, AccountHome.class);

    } catch (NamingException ne) {
      log("The client was unable to lookup the EJBHome.  Please make sure " +
      "that you have deployed the ejb with the JNDI name " + 
      "many2many.AccountHome on the WebLogic server at "+url);

      throw ne;
    }
  }

  /**
   * Look up the customer bean's home interface using JNDI.
   */
  private CustomerHome lookupCustomerHome()
    throws NamingException
  {
    Context ctx = getInitialContext();

    try {
      Object home = (CustomerHome) ctx.lookup("many2many.CustomerHome");
      return (CustomerHome) PortableRemoteObject.narrow(home, CustomerHome.class);

    } catch (NamingException ne) {
      log("The client was unable to lookup the EJBHome.  Please make sure " +
      "that you have deployed the ejb with the JNDI name " + 
      "many2many.CustomerHome on the WebLogic server at "+url);

      throw ne;
    }
  }

  /**
   * Get an initial context into the JNDI tree.
   *
   * Java2 clients can use the jndi.properties file to set the
   * INITIAL_CONTEXT_FACTORY and the PROVIDER_URL
   *  private Context getInitialContext() throws NamingException {
   *    return new InitialContext();
   *  }
   *
   *
   * Using a Properties object will work on JDK 1.1.x and Java2
   * clients
   */
  private Context getInitialContext() throws NamingException {

    try {
      // Get an InitialContext
      Properties h = new Properties();
      h.put(Context.INITIAL_CONTEXT_FACTORY,
          "weblogic.jndi.WLInitialContextFactory");
      h.put(Context.PROVIDER_URL, url);
      return new InitialContext(h);
    } catch (NamingException ne) {
      log("We were unable to get a connection to the WebLogic server at "+url);
      log("Please make sure that the server is running.");
      throw ne;
    }
  }

  private static void log(String s) {
    System.out.println(s);
  }

}







